#!/usr/bin/env python3
"""
Protocol Analyzer
Deep protocol analysis tool for network traffic.
Parameters:
    - interface: Network interface to monitor
    - protocol: Protocol to analyze (http, dns, smtp, etc.)
    - output: Output file for analysis results
Requires: scapy package and admin/root privileges
"""
from scapy.all import *
import argparse
import logging
import json
from datetime import datetime
import collections
import threading
import queue
import re

class ProtocolAnalyzer:
    def __init__(self, interface, protocol, output=None):
        self.interface = interface
        self.protocol = protocol.lower()
        self.output_file = output
        self.packet_queue = queue.Queue()
        self.stop_event = threading.Event()
        self.stats = collections.defaultdict(lambda: collections.Counter())
        self.setup_logging()
        self.setup_protocol_handlers()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def setup_protocol_handlers(self):
        self.protocol_handlers = {
            'http': self.analyze_http,
            'dns': self.analyze_dns,
            'smtp': self.analyze_smtp,
            'ftp': self.analyze_ftp,
            'ssh': self.analyze_ssh,
            'telnet': self.analyze_telnet,
            'all': self.analyze_all
        }

    def analyze_http(self, packet):
        if TCP in packet and (packet[TCP].dport == 80 or packet[TCP].sport == 80):
            payload = str(packet[TCP].payload)
            
            # Analyze HTTP methods
            for method in ['GET', 'POST', 'PUT', 'DELETE', 'HEAD']:
                if method in payload:
                    self.stats['http_methods'][method] += 1
            
            # Analyze HTTP headers
            if 'Host:' in payload:
                host = re.search(r'Host: (.*?)\r\n', payload)
                if host:
                    self.stats['http_hosts'][host.group(1)] += 1
            
            # Analyze response codes
            if 'HTTP/' in payload:
                code = re.search(r'HTTP/\d\.\d (\d{3})', payload)
                if code:
                    self.stats['http_responses'][code.group(1)] += 1

    def analyze_dns(self, packet):
        if DNS in packet:
            if packet.haslayer(DNSQR):
                qname = packet[DNSQR].qname.decode()
                self.stats['dns_queries'][qname] += 1
                self.stats['dns_types'][packet[DNSQR].qtype] += 1
            
            if packet.haslayer(DNSRR):
                rname = packet[DNSRR].rrname.decode()
                self.stats['dns_responses'][rname] += 1

    def analyze_smtp(self, packet):
        if TCP in packet and (packet[TCP].dport == 25 or packet[TCP].sport == 25):
            payload = str(packet[TCP].payload)
            
            for cmd in ['MAIL FROM:', 'RCPT TO:', 'DATA', 'QUIT']:
                if cmd in payload:
                    self.stats['smtp_commands'][cmd] += 1

    def analyze_ftp(self, packet):
        if TCP in packet and (packet[TCP].dport == 21 or packet[TCP].sport == 21):
            payload = str(packet[TCP].payload)
            
            for cmd in ['USER', 'PASS', 'LIST', 'RETR', 'STOR']:
                if cmd in payload:
                    self.stats['ftp_commands'][cmd] += 1

    def analyze_ssh(self, packet):
        if TCP in packet and (packet[TCP].dport == 22 or packet[TCP].sport == 22):
            self.stats['ssh_connections'][(packet[IP].src, packet[IP].dst)] += 1

    def analyze_telnet(self, packet):
        if TCP in packet and (packet[TCP].dport == 23 or packet[TCP].sport == 23):
            self.stats['telnet_connections'][(packet[IP].src, packet[IP].dst)] += 1

    def analyze_all(self, packet):
        if IP in packet:
            self.stats['ip_protocols'][packet[IP].proto] += 1
            
            if TCP in packet:
                self.stats['tcp_ports'][(packet[TCP].sport, packet[TCP].dport)] += 1
            elif UDP in packet:
                self.stats['udp_ports'][(packet[UDP].sport, packet[UDP].dport)] += 1

    def packet_callback(self, packet):
        if self.protocol in self.protocol_handlers:
            self.protocol_handlers[self.protocol](packet)
        
        if self.protocol == 'all':
            self.analyze_all(packet)

    def capture_packets(self):
        try:
            self.logger.info(f"Starting protocol analysis on interface {self.interface}")
            sniff(iface=self.interface, prn=self.packet_callback,
                  store=0, stop_filter=lambda _: self.stop_event.is_set())
        except Exception as e:
            self.logger.error(f"Error during packet capture: {str(e)}")

    def save_results(self):
        if self.output_file:
            try:
                results = {
                    'timestamp': datetime.now().isoformat(),
                    'interface': self.interface,
                    'protocol': self.protocol,
                    'statistics': {
                        name: dict(counter.most_common(10))
                        for name, counter in self.stats.items()
                    }
                }
                
                with open(self.output_file, 'w') as f:
                    json.dump(results, f, indent=4)
                self.logger.info(f"Analysis results saved to {self.output_file}")
            except Exception as e:
                self.logger.error(f"Error saving results: {str(e)}")

    def start_analysis(self):
        capture_thread = threading.Thread(target=self.capture_packets)
        capture_thread.start()
        
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            self.logger.info("Stopping protocol analysis...")
            self.stop_event.set()
            capture_thread.join()
            self.save_results()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Protocol Analyzer")
    parser.add_argument("interface", help="Network interface to monitor")
    parser.add_argument("--protocol", choices=['http', 'dns', 'smtp', 'ftp',
                                             'ssh', 'telnet', 'all'],
                      default='all', help="Protocol to analyze")
    parser.add_argument("--output", help="Output file for analysis results")
    args = parser.parse_args()

    analyzer = ProtocolAnalyzer(args.interface, args.protocol, args.output)
    analyzer.start_analysis()
